﻿using System;
using System.Collections.Generic;
using System.Linq;
using System.Text;
using System.Threading.Tasks;

namespace WSI_sample
{
    /*
     * This class represents a call entry in the call monitor.
     */
    class Call
    {
        private String caller;      // name (or number) of the caller
        private String called;      // name (or number) of the one being called
        private bool incoming;      // true, if it was an incoming call
        private long from;          // timestamp when the call was established
        private long to;            // timestamp when the call was ended
        private long timestamp;     // timestamp when the event was received

        public Call(String caller, String called, long timestamp)
        {
            this.caller = caller;
            this.called = called;
            this.from = 0;
            this.to = 0;
            this.timestamp = timestamp;
        }

        public override string ToString()
        {
            String str = "";
            if (incoming)
                str += "IN: from \"" + caller + "\" ";
            else
                str += "OUT: to \"" + called + "\" ";
            str += "at " + getTimestamp() + " - duration " + getDuration();

            return str;
        }

        private String getDuration()
        {
            if (from == 0)      // if from is 0, the call was not established
                return "00:00";

            // calculate the duration
            long duration = to - from;
            duration = duration / 10000;    // ticks to ms
            int seconds = (int)(duration / 1000) % 60;
            int minutes = (int)(duration / (1000 * 60)) % 60;
            
            String str = String.Format("{0}:{1}", minutes.ToString("D2"), seconds.ToString("D2"));

            return str;
        }

        private String getTimestamp()
        {
            DateTime date = new DateTime(timestamp);

            return date.ToString("HH:mm:ss");
        }

        public long From
        {
            get { return from; }
            set { from = value; }
        }

        public long To
        {
            get { return to; }
            set { to = value; }
        }

        public bool Incoming
        {
            set { incoming = value; }
        }
    }
}
