﻿using System;
using System.Collections.Generic;
using System.Linq;
using System.Text;
using System.Threading.Tasks;
using System.ComponentModel;
using System.Windows;
using System.Collections.ObjectModel;

namespace WSI_sample
{
    /*
     * This class is used to store the data. It uses the singleton pattern
     */
    class Data : INotifyPropertyChanged
    {
        private static Data instance;
        private String sessionId;       // gsSession ID received after login
        private String serverAddress;   // address of the server
        private String userId;          // ID of the user
        private String password;        // password from the user
        private String protocol;        // used protocol
        private String port;            // used port, automatically set when protocol is set
        private ObservableCollection<Message> listMessages; // list of all requests which were made to the WSI
        private ObservableCollection<Call> listCalls;       // list of all calls
        private bool loggedIn;          // true, if the user is logged in
        private bool tryLogin;          // true, when a login request was made but did not return yet


        private Data()
        {
            listMessages = new ObservableCollection<Message>();
            listCalls = new ObservableCollection<Call>();
            loggedIn = false;
        }

        public static Data getData()
        {
            if (instance == null)
                instance = new Data();
            return instance;
        }

        public void initData(String server, String user, String password, String protocol)
        {
            ServerAddress = server;
            UserId = user;
            Password = password;
            Protocol = protocol;
        }

        /*
         * Adds a message Element to the message list
         */
        public void addMessage(Message msg)
        {
            Application.Current.Dispatcher.Invoke(
                new Action(() => ListMessages.Add(msg)));
        }

        /*
         * Adds a call entry to the call list
         */
        public void addCall(Call call)
        {
            Application.Current.Dispatcher.Invoke(
                new Action(() => ListCalls.Add(call)));
        }

        public String getUrl()
        {
            return protocol + "://" + serverAddress + ":" + port;
        }

        public String SessionId
        {
            get { return sessionId; }
            set { sessionId = value; }
        }

        public String ServerAddress
        {
            get { return serverAddress; }
            set { serverAddress = value; }
        }

        public String UserId
        {
            get { return userId; }
            set { userId = value; }
        }

        public String Password
        {
            get { return password; }
            set { password = value; }
        }

        public String Protocol
        {
            get { return protocol; }
            set
            {
                protocol = value;
                if (value == "http")
                    Port = "8801";
                else
                    Port = "8802";
            }
        }

        public String Port
        {
            get { return port; }
            set { port = value; }
        }

        public ObservableCollection<Message> ListMessages
        {
            get { return listMessages; }
        }

        public ObservableCollection<Call> ListCalls
        {
            get { return listCalls; }
        }

        public bool LoggedIn
        {
            get { return loggedIn; }
            set
            {
                loggedIn = value;
                // fire events to update the buttons
                UpdateProperty("IsLoginEnabled");
                UpdateProperty("IsLogoutEnabled");
            }
        }

        public bool TryLogin
        {
            get { return tryLogin; }
            set
            {
                tryLogin = value;
                // fire events to update the buttons
                UpdateProperty("IsLoginEnabled");
                UpdateProperty("IsLogoutEnabled");
            }
        }

        // This property is to update the visibility of the login button.
        public bool IsLoginEnabled
        {
            get { return (!LoggedIn && !TryLogin); }
        }

        // This property is to update the visibility of the logout button.
        public bool IsLogoutEnabled
        {
            get { return (LoggedIn && !TryLogin); }
        }

        public event PropertyChangedEventHandler PropertyChanged;

        protected void UpdateProperty(string name)
        {
            if (PropertyChanged != null)
                PropertyChanged(this, new PropertyChangedEventArgs(name));
        }
    }
}
